      ****************************************************************
      *
      * Copyright (c) 2015 Ze'ev Atlas
      * Please refer to the LICENSE document to see all other
      * applicable copyrights.
      *
      *---------------------------------------------------------------
      * This COBOL copybook is free software; you can redistribute it 
      * and/or modify it under the terms of the GNU Lesser General 
      * Public License as published by the Free Software Foundation; 
      * either version 2.1 of the License, or (at your option) any 
      * later version.

      * This COBOL copybook is distributed in the hope that it will be 
      * useful, but WITHOUT ANY WARRANTY; without even the implied 
      * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
      * PURPOSE.  See the GNU Lesser General Public License for more 
      * details. 
      * (http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html)
      ****************************************************************
      * Adaptation of definitions for constants for the regular
      * expression library syntax for COBOL.  It is not a necessary
      * copybook and it is GNU oriented and most comments are from
      * the GNU implementation.  I debated if this should at all be 
      * published, and if so, under what license.  In the end, I 
      * decided to publish this particular copybook under the same 
      * GNU Lesser General Public License as originally published by 
      * the Free Software Foundation.
      ****************************************************************
      * reg_syntax_t is PIC 9(9) COMP-5.
      * GNU implementation specific:
      * Determine the regexp syntax.  Defaults to Emacs syntax (0).  
      * The bits are given in alphabetical order.
       01  REGEX-SYNTAX-CONSTANTS.
      * When not set, then \ inside a bracket expression is a literal.
      * When set, then such a \ quotes the following character.
           05 RE-BACKSLASH-ESCAPE-IN-LISTS  PIC 9(9) COMP-5 VALUE 1.
      * When not set, then + and ? are operators, and \+ and \? are 
      * literals.  When set, then \+ and \? are operators and + and ? 
      * are literals.  
           05 RE-BK-PLUS-QM                 PIC 9(9) COMP-5 VALUE 2.
      * When set, then character classes are supported.  They are:    
      * [:alpha:], [:upper:], [:lower:],  [:digit:], [:alnum:], 
      * [:xdigit:], [:space:], [:print:], [:punct:], [:graph:], and 
      * [:cntrl:].  When not set, then character classes are not    
      * supported.  
           05 RE-CHAR-CLASSES               PIC 9(9) COMP-5 VALUE 4.
      * When set, then ^ and $ are always anchors (outside bracket 
      * expressions, of course).  When not set, then it depends:
      * ^  is an anchor if it is at the beginning of a regular
      *    expression or after an open-group or an alternation 
      *    operator;
      * $  is an anchor if it is at the end of a regular expression, 
      *    or before a close-group or an alternation operator.

      * This bit could be (re)combined with RE_CONTEXT_INDEP_OPS, 
      * because POSIX draft 11.2 says that * etc. in leading 
      * positions is undefined.  We already implemented a previous 
      * draft which made those constructs invalid, though, so we 
      * haven't changed the code back.  
           05 RE-CONTEXT-INDEP-ANCHORS      PIC 9(9) COMP-5 VALUE 8.
      * When set, then special characters are always special
      * regardless of where they are in the pattern.  When not set,  
      * then special characters are special only in some contexts; 
      * otherwise they are ordinary.  Specifically, * + ? and 
      * intervals are only special when not after the beginning,
      * open-group, or alternation operator.  
           05 RE-CONTEXT-INDEP-OPS         PIC 9(9) COMP-5 VALUE 16.
      * When set, then *, +, ?, and { cannot be first in an re or 
      * immediately after an alternation or begin-group operator.
           05 RE-CONTEXT-INVALID-OPS       PIC 9(9) COMP-5 VALUE 32.	  
      * When set, then . matches newline.  When not set, then 
      * it doesn't.  
           05 RE-DOT-NEWLINE               PIC 9(9) COMP-5 VALUE 64.
      * When set, then . doesn't match NUL.  When not set,  
      * then it does.  
           05 RE-DOT-NOT-NULL             PIC 9(9) COMP-5 VALUE 128.
      * When set, nonmatching lists [^...] do not match 
      * newline.  When not set, they do match.  
           05 RE-HAT-LISTS-NOT-NEWLINE    PIC 9(9) COMP-5 VALUE 256.
      * When set, either \{...\} or {...} defines an 
      * interval, depending on RE_NO_BK_BRACES.  When not set, \{, \}, 
      * {, and } are literals.  
           05 RE-INTERVALS                PIC 9(9) COMP-5 VALUE 512.
      * When set, +, ? and | aren't recognized as operators.
      * When not set, they are.  
           05 RE-LIMITED-OPS             PIC 9(9) COMP-5 VALUE 1024.
      * When set, newline is an alternation operator.
      * When not set, newline is literal.  
           05 RE-NEWLINE-ALT             PIC 9(9) COMP-5 VALUE 2048.
      * When set, then `{...}' defines an interval, and \{ 
      * and \} are literals.  When not set, then `\{...\}' defines an 
      * interval.  
           05 RE-NO-BK-BRACES            PIC 9(9) COMP-5 VALUE 4096.
      * When set, (...) defines a group, and \( and \) are 
      * literals.  When not set, \(...\) defines a group, and ( and ) 
      * are literals.  
           05 RE-NO-BK-PARENS            PIC 9(9) COMP-5 VALUE 8192.
      * When set, then \<digit> matches <digit>.  When not set, then 
      * \<digit> is a back-reference.  
           05 RE-NO-BK-REFS             PIC 9(9) COMP-5 VALUE 16384.
      * When set, then | is an alternation operator, and \| is 
      * literal.  When not set, then \| is an alternation operator, 
      * and | is literal.  
           05 RE-NO-BK-VBAR             PIC 9(9) COMP-5 VALUE 32768.	  
      * When set, then an ending range point collating higher than
      * the starting range point, as in [z-a], is invalid.  When not 
      * set, then when ending range point collates higher than the
      * starting range point, the range is ignored.  
           05 RE-NO-EMPTY-RANGES        PIC 9(9) COMP-5 VALUE 65536.
      * When set, then an unmatched ) is ordinary.  When not set, then
      * an unmatched ) is invalid.  
           05 RE-UNMATCHED-RIGHT-PAREN-ORD PIC 9(9) COMP-5 
                                           VALUE 131072.
      * When set, succeed as soon as we match the whole pattern, 
      * without further backtracking.  
           05 RE-NO-POSIX-BACKTRACKING     PIC 9(9) COMP-5 
                                           VALUE 262144.
      * When set, do not process the GNU regex operators.  When not 
      * set, then the GNU regex operators are recognized. 
           05 RE-NO-GNU-OPS                PIC 9(9) COMP-5 
                                           VALUE 524288.
      * When set, turn on internal regex debugging.  When not set, and 
      * debugging was on, turn it off.  This only works if regex.c is 
      * compiled -DDEBUG.  We define this bit always, so that all 
      * that's needed to turn on debugging is to recompile regex.c; 
      * the calling code can always have this bit set, and it won't 
      * affect anything in the normal case. 
           05 RE-DEBUG                     PIC 9(9) COMP-5 
                                           VALUE 1048576.
      * When set, a syntactically invalid interval is treated as a
      * string of ordinary characters.  For example, the ERE 'a{1' is
      * treated as 'a\{1'.  
           05 RE-INVALID-INTERVAL-ORD      PIC 9(9) COMP-5 
                                           VALUE 2097152.
      * When set, then ignore case when matching.  When not set, then 
      * case is significant.  
           05 RE-ICASE                     PIC 9(9) COMP-5 
                                           VALUE 4194304.
      * This bit is used internally like RE_CONTEXT_INDEP_ANCHORS but 
      * only for ^, because it is difficult to scan the regex 
      * backwards to find whether ^ should be special.  
           05 RE-CARET-ANCHORS-HERE        PIC 9(9) COMP-5 
                                           VALUE 8388608.
      * When set, then \{ cannot be first in an bre or immediately 
      * after an alternation or begin-group operator.  
           05 RE-CONTEXT-INVALID-DUP       PIC 9(9) COMP-5 
                                           VALUE 16777216.	  
      * When set, then no_sub will be set to 1 during
      * re_compile_pattern.  
           05 RE-NO-SUB                    PIC 9(9) COMP-5 
 
      * Define combinations of the above bits for the standard 
      * possibilities.
      * In COBOL we really don't care for AWK, EMACS, GREP, EGREP, ED
      * SED or any of their combinations thereof

      * Syntax bits common to both basic and extended POSIX regex 
      * syntax.  
      * #define _RE_SYNTAX_POSIX_COMMON						\
      * (RE_CHAR_CLASSES | RE_DOT_NEWLINE      | RE_DOT_NOT_NULL \
      * | RE_INTERVALS  | RE_NO_EMPTY_RANGES) 
      * 4+64+128+512+65536=66244

      * #define RE_SYNTAX_POSIX_BASIC						\
      * (_RE_SYNTAX_POSIX_COMMON | RE_BK_PLUS_QM | 
      * RE_CONTEXT_INVALID_DUP) /* 66244+2+16777216=16843462 
           05 RE-SYNTAX-POSIX-BASIC        PIC 9(9) COMP-5 
                                           VALUE 16843462.	
      * Differs from ..._POSIX_BASIC only in that RE_BK_PLUS_QM 
      * becomes RE_LIMITED_OPS, i.e., \? \+ \| are not recognized.  
      * Actually, this isn't minimal, since other operators, such as 
      * \`, aren't disabled.  
           05 RE-SYNTAX-POSIX-MINIMAL-BASIC PIC 9(9) COMP-5 
                                           VALUE 16843462.	
      * #define RE_SYNTAX_POSIX_MINIMAL_BASIC					\
      * (_RE_SYNTAX_POSIX_COMMON | RE_LIMITED_OPS)
      * 66244+1024=67268 
           05 RE-SYNTAX-POSIX-EXTENDED     PIC 9(9) COMP-5 
                                           VALUE 8631028.	
      * #define RE_SYNTAX_POSIX_EXTENDED					\
      * (_RE_SYNTAX_POSIX_COMMON  | RE_CONTEXT_INDEP_ANCHORS \
      * | RE_CONTEXT_INDEP_OPS   | RE_NO_BK_BRACES				\
      * | RE_NO_BK_PARENS        | RE_NO_BK_VBAR				\
      * | RE_CONTEXT_INVALID_OPS | RE_UNMATCHED_RIGHT_PAREN_ORD)
      * 66244+8388608+16+4096+8192+32768+32+131072=8631028 

      * Differs from ..._POSIX_EXTENDED in that RE_CONTEXT_INDEP_OPS
      * is removed and RE_NO_BK_REFS is added.  
           05 RE-SYNTAX-POSIX-MINIMAL-EXTENDED PIC 9(9) COMP-5 
                                           VALUE 8647396.	
      * #define RE_SYNTAX_POSIX_MINIMAL_EXTENDED				\
      * (_RE_SYNTAX_POSIX_COMMON  | RE_CONTEXT_INDEP_ANCHORS	\
      * | RE_CONTEXT_INVALID_OPS | RE_NO_BK_BRACES				\
      * | RE_NO_BK_PARENS        | RE_NO_BK_REFS				\
      * | RE_NO_BK_VBAR	    | RE_UNMATCHED_RIGHT_PAREN_ORD)
      * 66244+8388608+4096+8192+32768+32+131072+16384=8647396 
